// -----------------------------------------------------------------------------
// (C) Bibix
// The content below includes confidential, proprietary information of
// Bibix. All use, disclosure, and/or reproduction is prohibited
// unless authorized in writing. All rights reserved.
// -----------------------------------------------------------------------------
// File         : tb_cascade.cpp
// Description  : C-testbench for "cascade" demo
// Author       : Sabih Gerez, Bibix
// Creation date: August 25, 2010
// -----------------------------------------------------------------------------
// $Rev: 47 $
// $Author: sabih $
// $Date: 2011-02-03 01:25:20 +0100 (Thu, 03 Feb 2011) $
// $Log$
// -----------------------------------------------------------------------------

// include global files
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

// include local files
#include "cascade.h"

int main(int argc, char* argv[])
{
  // open input file
  FILE *infile;

  infile = fopen("cascade.in", "rt");
  if (infile == NULL)
     printf("Cannot open cascade.in for reading!\n");
  assert(infile != NULL);

  // open output file
  FILE *outfile;

  outfile = fopen("cascade.out", "wt");
  if (outfile == NULL)
     printf("Cannot open cascade.out for writing!\n");
  assert(outfile != NULL);

  // set up hardware
  cascade hardware;

  // reset
  hardware.reset();

  // no more than 10 characters per line in input file
  char line[10];
  int in_number;
  i32 out_data;

  // static variable on behalf of a periodic clear
  static int input_counter = 0;
  
  while (fgets(line, 10, infile) != NULL)  {
    // calculate periodic clear
    int local_clear;
    if (input_counter++ == 10) {
       input_counter = 0;
       local_clear = 1;
    }
    else
       local_clear = 0;
      
    sscanf(line, "%d", &in_number);
    hardware.run(u32(in_number), out_data, local_clear);
    fprintf(outfile, "%6d %6d\n", in_number, (int)out_data);
  }

  // close files
  fclose(infile);
  fclose(outfile);
  printf("OK! End of input file reached. Stopping simulation.\n");
}
