// -----------------------------------------------------------------------------
// (C) Bibix
// The content below includes confidential, proprietary information of
// Bibix. All use, disclosure, and/or reproduction is prohibited
// unless authorized in writing. All rights reserved.
// -----------------------------------------------------------------------------
// File         : arx_util.cpp
// Description  : Arx utility functions
// Author       : Klaas Hofstra, University of Twente
// Creation date: June 29, 2010 (Header added by SHG)
// -----------------------------------------------------------------------------
// $Rev: 49 $
// $Author: sabih $
// $Date: 2011-02-03 10:46:15 +0100 (Thu, 03 Feb 2011) $
// $Log$
// -----------------------------------------------------------------------------

#include "arx_util.h"

/*
 * 2006-07-30
 * - replaced types with i32,u32,i64,u64
 * - now rounding to nearby int in converions
 * - introduced a 64-bit and 32-bit version of double_to_signed
 * 2006-08-14
 * - replaced std::endl with \n to prevent flushing
 * 2006-09-15
 * - fixed clamping error in to_signed()
 */

i32 double_to_signed32(double d, int wl, int iwl) {
	i32 out;
	to_signed(d, out, wl, iwl);
	return out;
}

i64 double_to_signed64(double d, int wl, int iwl) {
	i64 out;
	to_signed(d, out, wl, iwl);
	return out;
}

double quant_double(double in, int wl, int iwl)
{
	double max = signed_max(wl, iwl);
	double min = signed_min(iwl);
	/* clamp */
	if (in >= max) {
		return max;
	}
	else if (in <= min) {
		return min;
	}
	double tmp;
	int fwl = wl - iwl;
	if (fwl >= 0) {
		tmp = nearbyint(shl_double(in, fwl));
		tmp = shr_double(tmp, fwl);
	}
	else {
		tmp = nearbyint(shr_double(in, fwl));
		tmp = shl_double(tmp, fwl);
	}
	return tmp;
}

void output_vcd_value(std::ofstream &f, bool value, char* name)
{
	if (value)
		f << '1';
	else
		f << '0';
	f << name << "\n";
}

void output_vcd_value(std::ofstream &f, int value, char* name)
{
	if (value == 1)
		f << '1';
	else
		f << '0';
	f << name << "\n";
}

void output_vcd_value(std::ofstream &f, unsigned int value, char* name)
{
	if (value == 1)
		f << '1';
	else
		f << '0';
	f << name << "\n";
}

void output_vcd_value(std::ofstream &f, float value, char* name)
{
	f << 'X' << name << "\n";
}

char* down_scope(std::ofstream &f, char* name)
{
	char *p;
	char *tmp = name;
	while ((p = strstr(tmp, "__")) != NULL) {
		f << "$scope module ";
		while (tmp != p) {
			f << *tmp;
			tmp++;
		}
		f << " $end\n";
		tmp += 2;
	}
	return tmp;
}

void up_scope(std::ofstream &f, char* name)
{
	char *p;
	char *tmp = name;
	while ((p = strstr(tmp, "__")) != NULL) {
		f << "$upscope $end\n";
		tmp = p + 2;
	}
}

void init_vcd_value(std::ofstream &f, char* vcdName, char* name)
{
	char* tmp = down_scope(f, name);
	f << "$var wire 1 " << vcdName << ' ' << tmp << " $end\n";
	up_scope(f, name);
}

void init_vcd_vector(std::ofstream &f, int width, char* vcdName, char* name)
{
	char* tmp = down_scope(f, name);
	f << "$var wire " << width << ' ' << vcdName << ' ' << tmp << " [" << width-1 << ":0] $end\n";
	up_scope(f, name);
}


