// -----------------------------------------------------------------------------
// (C) Bibix
// The content below includes confidential, proprietary information of
// Bibix. All use, disclosure, and/or reproduction is prohibited
// unless authorized in writing. All rights reserved.
// -----------------------------------------------------------------------------
// File         : arx_util.h
// Description  : Header file for Arx utility functions
// Author       : Klaas Hofstra, University of Twente
// Creation date: June 29, 2010 (Header added by SHG)
// -----------------------------------------------------------------------------
// $Rev: 49 $
// $Author: sabih $
// $Date: 2011-02-03 10:46:15 +0100 (Thu, 03 Feb 2011) $
// $Log$
// -----------------------------------------------------------------------------

#ifndef ARX_UTIL_H
#define ARX_UTIL_H

#include <cstring>
#include <cmath>
#include "arx_types.h"
#include <iostream>
#include <fstream>

void output_vcd_value(std::ofstream &f, bool value, char* name);
void output_vcd_value(std::ofstream &f, int value, char* name);
void output_vcd_value(std::ofstream &f, unsigned int value, char* name);
void output_vcd_value(std::ofstream &f, float value, char* name);
void init_vcd_value(std::ofstream &f, char* vcdName, char* name);
void init_vcd_vector(std::ofstream &f, int width, char* vcdName, char* name);

i32 double_to_signed32(double d, int wl, int iwl);
i64 double_to_signed64(double d, int wl, int iwl);
double quant_double(double in, int wl, int iwl);

inline double shl_double(double in, int shift)
{
	if (shift > 32) {
		return in * (double)(u64(1) << shift);
	}
	else {
		return in * (double)(u32(1) << shift);
	}
}

inline double shr_double(double in, int shift)
{
	if (shift > 32) {
		return in / (double)(u64(1) << shift);
	}
	else {
		return in / (double)(u32(1) << shift);
	}
}

inline double pow2(int p)
{
	if (p >= 0) {
		return shl_double(1.0, p);
	}
	else {
		return shr_double(1.0, p);
	}
}

inline double signed_max(int wl, int iwl)
{
	return pow2(iwl-1) - pow2(-(wl-iwl));
}

inline double signed_min(int iwl)
{
	return -pow2(iwl-1);
}

template <typename T>
void to_signed(double in, T &out, int wl, int iwl) {
	int fwl = wl - iwl;
	if (fwl >= 0) {
		out = (T)nearbyint(shl_double(in, fwl));
	}
	else {
		out = (T)nearbyint(shr_double(in, fwl));
	}
	/* clamp */
	T max = (T(1) << (wl-1)) - 1;
	T min = -(T(1) << (wl-1));
	if (out > max) {
		out = max;
	}
	else if (out < min) {
		out = min;
	}
}

template <typename T>
void to_double(const T in, double &out, int wl, int iwl)
{
	int fwl =  wl - iwl;
	double s;
	if (fwl > 0) {
		out = shr_double((double)in, fwl);
	}
	else {
		out = shl_double((double)in, -fwl);
	}
}

template <typename T>
double signed_to_double(const T in, int wl, int iwl)
{
    int fwl =  wl - iwl;
    double s;
    if (fwl > 0) {
		s = shr_double((double)in, fwl);
    }
    else if (fwl < 0){
		s = shl_double((double)in, -fwl);
    }
    else {
        s = (double)in;
    }
    return s;
}

template <typename T>
void output_vcd_vector(std::ofstream &f, T vector, int bits, char* name)
{
    T mask = 1;
    mask = mask << (bits-1);
    f << 'b';
    while (mask != 0) {
        if ((vector & mask) == 0)
            f << '0';
        else
            f << '1';
        mask = mask >> 1;
    }
    f << ' ' << name << "\n";
}

#endif

